@@ -12,19 +12,20 @@ the next time zim is started without arguments.
 @newfield column: Column, Columns
 '''
 
+import os
 import gtk
 import pango
 import logging
 
-from zim.fs import *
-from zim.notebook import get_notebook_list, init_notebook, NotebookInfo
+from zim.fs import File, Dir
+from zim.notebook import get_notebook_list, get_notebook_info, init_notebook, NotebookInfo
 from zim.config import data_file
-from zim.gui.widgets import ui_environment, Dialog, IconButton, _encode_xml
+from zim.gui.widgets import ui_environment, Dialog, IconButton, encode_markup_text
 
 logger = logging.getLogger('zim.gui.notebookdialog')
 
 
-OPEN_COL = 0   # column with boolean if notebook is open alreadys
+OPEN_COL = 0   # column with boolean if notebook is open already
 NAME_COL = 1   # column with notebook name
 TEXT_COL = 2   # column with a formatted containing name and path
 PIXBUF_COL = 3 # column containing the notebook icon
@@ -33,7 +34,7 @@ INFO_COL = 4   # column with the NotebookInfo object
 
 def prompt_notebook():
 	'''Prompts the NotebookDialog and returns the result or None.
-	As a special case for first time usage it immediatly prompts for
+	As a special case for first time usage it immediately prompts for
 	the notebook location without showing the notebook list.
 	'''
 	list = get_notebook_list()
@@ -47,7 +48,7 @@ def prompt_notebook():
 			list.write()
 			return dir.uri
 		else:
-			return None # User cancelled the dialog ?
+			return None # User canceled the dialog ?
 	else:
 		# Multiple notebooks defined and no default
 		return NotebookDialog(ui=None).run()
@@ -56,11 +57,11 @@ def prompt_notebook():
 class NotebookTreeModel(gtk.ListStore):
 	'''TreeModel that wraps a notebook list
 
-	@column OPEN_COL: boolean, True if the notebook is opened already
-	@column NAME_COL: string, name of the notebook
-	@column TEXT_COL: string, formatted string containg the name and path
-	@column PIXBUF_COL: gtk.gdk.Pixbuf, the icon of the notebook (if any)
-	@column INFO_COL: L{NotebookInfo} object
+	@column: C{OPEN_COL}: boolean, True if the notebook is opened already
+	@column: C{NAME_COL}: string, name of the notebook
+	@column: C{TEXT_COL}: string, formatted string containg the name and path
+	@column: C{PIXBUF_COL}: gtk.gdk.Pixbuf, the icon of the notebook (if any)
+	@column: C{INFO_COL}: L{NotebookInfo} object
 
 	@note: To refer to the notebook in an unambiguous way, use the uri stored
 	in the L{NotebookInfo} object.
@@ -109,7 +110,7 @@ class NotebookTreeModel(gtk.ListStore):
 	def _append(self, info):
 		path = File(info.uri).path
 		text = '<b>%s</b>\n<span foreground="#5a5a5a" size="small">%s</span>' % \
-				(_encode_xml(info.name), _encode_xml(path))
+				(encode_markup_text(info.name), encode_markup_text(path))
 				# T: Path label in 'open notebook' dialog
 
 		if info.icon and File(info.icon).exists():
@@ -180,10 +181,13 @@ class NotebookComboBox(gtk.ComboBox):
 	'''Combobox showing the a list of notebooks'''
 
 	def __init__(self, model=None, current=None):
-		'''Constructor, "model" should be a NotebookTreeModel or None to
-		use the default list. The notebook 'current' will be shown in the
-		widget - if it is not in the list it wil be added. Otherwise the default
-		will be shown.
+		'''Constructor,
+
+		@param model: either a L{NotebookTreeModel} or C{None} to use
+		the default list.
+		@param current: uri, C{Dir}, C{NotebookInfo}, or C{Notebook}
+		object for the current notebook. If C{None} the default
+		notebook will be shown (if any).
 		'''
 		if model is None:
 			model = NotebookTreeModel()
@@ -198,8 +202,11 @@ class NotebookComboBox(gtk.ComboBox):
 			self.set_default_active()
 
 	def set_default_active(self):
-		'''Select the default notebook in the combobox'''
-		iter = self.get_model().get_iter_for_default()
+		'''Select the default notebook in the combobox or clear the
+		combobox if no default notebook was defined.
+		'''
+		model = self.get_model()
+		iter = model.get_iter_for_default()
 		if iter is None:
 			self.set_active(-1)
 		else:
@@ -207,8 +214,12 @@ class NotebookComboBox(gtk.ComboBox):
 
 	def set_notebook(self, uri, append=False):
 		'''Select a specific notebook in the combobox.
-		If 'append' is True it will appended if it didn't exist yet
-		in the notebook list.
+
+		@param uri: uri, C{Dir}, C{NotebookInfo}, or C{Notebook}
+		object for a notebook (string or any object with an C{uri}
+		property)
+		@param append: if C{True} the notebook will appended to the list
+		if it was not listed yet.
 		'''
 		if isinstance(uri, basestring):
 			assert uri.startswith('file://')
@@ -259,7 +270,7 @@ class NotebookDialog(Dialog):
 	of defined notebooks.
 
 	Can either be run modal using run(), in which case the selected
-	notebook is returned (or None when the dialog is cancelled).
+	notebook is returned (or None when the dialog is canceled).
 	To run this dialog non-model a callback needs to be specified
 	which will be called with the path for the selected notebook.
 	'''
@@ -378,12 +389,18 @@ class AddNotebookDialog(Dialog):
 		label.set_alignment(0.0, 0.5)
 		self.vbox.pack_start(label, False)
 
-		if name is None and folder is None:
+		self._name_set = not name is None
+		self._folder_set = not folder is None
+
+		if ui_environment['platform'] == 'maemo':
+			nb_folder = '~/MyDocs/Notebooks/' # 'MyDocs' is the "Device" folder on maemo
+		else:
+			nb_folder = '~/Notebooks/'
+
+		if not self._name_set and not self._folder_set:
 			name = 'Notes'
-			if ui_environment['platform'] == 'maemo':
-				folder = '~/MyDocs/Notes' # 'MyDocs' is the "Device" folder on maemo
-			else:
-				folder = '~/Notes'
+			folder = nb_folder + name
+		# else set below by _changed methods
 
 		self.add_form((
 			('name', 'string', _('Name')), # T: input field in 'Add Notebook' dialog
@@ -391,13 +408,59 @@ class AddNotebookDialog(Dialog):
 		), {
 			'name': name,
 			'folder': folder,
-		} )
+		})
 
 		self.add_help_text('''\
 To create a new notebook you need to select an empty folder.
 Of course you can also select an existing zim notebook folder.
 ''') # T: help text in the 'Add Notebook' dialog
 
+		# Hook entries to copy name when appropriate
+		self._block_update = False
+		self.on_name_changed(None, interactive=False)
+		self.on_folder_changed(None, interactive=False)
+		self.form.widgets['name'].connect('changed', self.on_name_changed)
+		self.form.widgets['folder'].connect('changed', self.on_folder_changed)
+
+	def on_name_changed(self, o, interactive=True):
+		# When name is changed, update folder accordingly
+		# unless the folder was set explicitly already
+		if self._block_update: return
+		self._name_set = self._name_set or interactive
+		if self._folder_set: return
+
+		name = self.form.widgets['name'].get_text()
+		folder = self.form.widgets['folder'].get_text()
+		dir = os.path.dirname(folder).strip('/\\')
+
+		self._block_update = True
+		self.form.widgets['folder'].set_text(os.path.join(dir, name))
+		self._block_update = False
+
+	def on_folder_changed(self, o, interactive=True):
+		# When folder is changed, update name accordingly
+		if self._block_update: return
+		self._folder_set = self._folder_set or interactive
+
+		# Check notebook info (even when name was set already)
+		if interactive or not self._name_set:
+			folder = self.form['folder']
+			if folder and folder.exists():
+				info = get_notebook_info(folder)
+				if info: # None when no config found
+					self._block_update = True
+					self.form['name'] = info.name
+					self._block_update = False
+					return
+
+		# Else use basename unless the name was set explicitly already
+		if self._name_set: return
+
+		folder = self.form.widgets['folder'].get_text().strip('/\\')
+		self._block_update = True
+		self.form['name'] = os.path.basename(folder)
+		self._block_update = False
+
 	def do_response_ok(self):
 		name = self.form['name']
 		folder = self.form['folder']
